{-# LANGUAGE OverloadedStrings #-}

module Format where

import Types

import qualified Data.ByteString as B
import qualified Data.ByteString.Builder as BB
import qualified Data.ByteString.Internal as BI
import qualified Data.ByteString.Lazy as BL
import Data.String
import Substr

pprHunkHdr i j = fromString $ "@@ -" ++ show i ++ " +" ++ show j ++ " @@" 

lineSep = fromString "\n"

pprHunks :: [Hunk] -> BB.Builder
pprHunks = mconcat . map pprHunk

pprHunk :: Hunk -> BB.Builder
pprHunk ((i, j), toks) = mconcat ((pprHunkHdr i j <> lineSep) : map pprDiff1 toks)

pprDiff1 :: (Op, Tok) -> BB.Builder
pprDiff1 (op, (tok, s)) =
  fromString pfx <> escNewlines s <> lineSep
  where
    pfx = [opc,tc]
    opc = case op of
      Add -> '+'
      Keep -> ' '
      Remove -> '-'
      MineChanged -> '<'
      Original -> '='
      YourChanged -> '>'
    tc = if tok then '|' else '.'

escNewlines :: BS -> BB.Builder
escNewlines s
  | B.null s = mempty
  | B.head s == BI.c2w '\n' = BB.stringUtf8 "\\n" <> escNewlines (B.tail s)
  | B.head s == BI.c2w '\\' = BB.stringUtf8 "\\\\" <> escNewlines (B.tail s)
  | otherwise = BB.word8 (B.head s) <> escNewlines (B.tail s)
